//+------------------------------------------------------------------+
//|                                            PivotCandlesClass.mqh |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2012, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
//+------------------------------------------------------------------+
//| Input parameters                                                 |
//+------------------------------------------------------------------+
input int      iMaxBodySize            = 50;  //   , %
input int      iMaxShadowSize          = 0;   //     , %
input int      iVolatilityCandlesCount = 5;   //       
input int      iPrevCandlesCount       = 5;   //     ,    
input int      iVolatilityPercent      = 100; //      , %
input int      iMAPeriod               = 10;  //     
//+------------------------------------------------------------------+
//| Class definition                                                 |
//+------------------------------------------------------------------+
class CPivotCandlesClass
  {
private:
   MqlRates          m_candles[];              //       
   int               m_history_depth;          //     
   int               m_handled_candles_count;  //    
   
   double            m_ma_value;               //     
   double            m_prev_ma_value;          //     
   bool              m_is_highest;             //     
   bool              m_is_lowest;              //     
   double            m_volatility;             //  
   int               m_candle_pattern;         // n  
   
   void              PrepareArrayForNewCandle();        //      
   int               CheckCandleSize(MqlRates& candle); //     
   void              PrepareCalculation();
protected:
   int               DoAnalizeNewCandle();              //  
public:
   void              CPivotCandlesClass(); 
   
   void              CleanupHistory();                  //      
   double            MAValue() {return m_ma_value;}     //    
   int               AnalizeNewCandle(MqlRates& candle);
   int               AnalizeNewCandle( const datetime time,
                                       const double open,
                                       const double high,
                                       const double low,
                                       const double close,
                                       const long tick_volume,
                                       const long volume,
                                       const int spread );
  };
//+------------------------------------------------------------------+
//| CPivotCandlesClass                                               |
//+------------------------------------------------------------------+
//|                                               |
//+------------------------------------------------------------------+ 
void CPivotCandlesClass::CPivotCandlesClass()
  {
   //       
   m_history_depth = (int)MathMax(MathMax(
      iVolatilityCandlesCount + 1, iPrevCandlesCount + 1), iMAPeriod);
   m_handled_candles_count = 0;
   m_prev_ma_value = 0;
   m_ma_value = 0;
   
   ArrayResize(m_candles, m_history_depth);
  }  
//+------------------------------------------------------------------+
//| CleanupHistory                                                   |
//+------------------------------------------------------------------+
//|                               |
//+------------------------------------------------------------------+
void CPivotCandlesClass::CleanupHistory()
  {
   //  
   ArrayFree(m_candles);
   ArrayResize(m_candles, m_history_depth);
   
   //   
   m_handled_candles_count = 0;
   m_prev_ma_value = 0;
   m_ma_value = 0;   
  }
//+------------------------------------------------------------------+
//| AnalizeNewCandle                                                 |
//+------------------------------------------------------------------+
//|                             |
//|                       |
//+------------------------------------------------------------------+
int CPivotCandlesClass::AnalizeNewCandle( const datetime time,
                                          const double open,
                                          const double high,
                                          const double low,
                                          const double close,
                                          const long tick_volume,
                                          const long volume,
                                          const int spread )
  {
   //      
   PrepareArrayForNewCandle();

   //    
   m_candles[0].time          = time;
   m_candles[0].open          = open;
   m_candles[0].high          = high;
   m_candles[0].low           = low;
   m_candles[0].close         = close;
   m_candles[0].tick_volume   = tick_volume;
   m_candles[0].real_volume   = volume;
   m_candles[0].spread        = spread;

   //     ?
   if (m_handled_candles_count < m_history_depth)
      return 0;
   else
      return DoAnalizeNewCandle();
  }  
//+------------------------------------------------------------------+
//| AnalizeNewCandle                                                 |
//+------------------------------------------------------------------+
//|                             |
//|                                           |
//+------------------------------------------------------------------+
int CPivotCandlesClass::AnalizeNewCandle(MqlRates& candle)
  {
   //         
   PrepareArrayForNewCandle();

   //   
   m_candles[0] = candle;

   //     ?
   if (m_handled_candles_count < m_history_depth)
      return 0;
   else
      return DoAnalizeNewCandle();
  }
//+------------------------------------------------------------------+
//| PrepareArrayForNewCandle                                         |
//+------------------------------------------------------------------+ 
//|                                |
//+------------------------------------------------------------------+ 
void CPivotCandlesClass::PrepareArrayForNewCandle()
  {
   //     ,     
   ArrayCopy(m_candles, m_candles, 1, 0, m_history_depth-1);
   
   //    
   m_handled_candles_count++;
  }
//+------------------------------------------------------------------+
//| CalcMAValue                                                      |
//+------------------------------------------------------------------+ 
//|     ,         |
//|                                          |
//+------------------------------------------------------------------+ 
void CPivotCandlesClass::PrepareCalculation()
  {
   //   
   m_prev_ma_value = m_ma_value;
   m_ma_value = 0;
   
   m_is_highest = true; //     
   m_is_lowest = true;  //     
   m_volatility = 0;  //  
   
   double price_sum = 0; //    
   for (int i=0; i<m_history_depth; i++)
     {
      if (i<iMAPeriod)
         price_sum += m_candles[i].close;
      if (i>0 && i<=iVolatilityCandlesCount)
         m_volatility += m_candles[i].high - m_candles[i].low;
      if (i>0 && i<=iPrevCandlesCount)
        {
         m_is_highest = m_is_highest && (m_candles[0].high > m_candles[i].high);
         m_is_lowest = m_is_lowest && (m_candles[0].low < m_candles[i].low);
        }
     }
   m_ma_value = price_sum / iMAPeriod;
   m_volatility /= iVolatilityCandlesCount;
   
   m_candle_pattern = CheckCandleSize(m_candles[0]);
  }
//+------------------------------------------------------------------+
//| CheckCandleSize                                                  |
//+------------------------------------------------------------------+
//| ,                     |
//|   :                                          |
//|   0 -                           |
//|   1 -   ""                                     |
//|   -1 -   " "                          |
//+------------------------------------------------------------------+ 
int CPivotCandlesClass::CheckCandleSize(MqlRates &candle)
  {
   double candle_height=candle.high-candle.low;          //   
   double candle_body=MathAbs(candle.close-candle.open); //   

   //   ,     
   if(candle_height==0) return 0; 
   if(candle_body/candle_height*100.0>iMaxBodySize)
      return 0;

   double candle_top_shadow=candle.high-MathMax(candle.open,candle.close);   //    
   double candle_bottom_shadow=MathMin(candle.open,candle.close)-candle.low; //    

   //     ,  -  ""
   if(candle_top_shadow/candle_height*100.0<=iMaxShadowSize)
      return 1;
   //     ,  -  " "
   else if(candle_bottom_shadow/candle_height*100.0<=iMaxShadowSize)
      return -1;
   else
      return 0;
  }
//+------------------------------------------------------------------+
//| DoAnalizeNewCandle                                               |
//+------------------------------------------------------------------+
//|                                 |
//+------------------------------------------------------------------+ 
int CPivotCandlesClass::DoAnalizeNewCandle()
  {
   //      
   PrepareCalculation();
   
   //       
   int signal = 0;
   
   ///////////////////////////////////////////////////////////////////
   //                                         //
   ///////////////////////////////////////////////////////////////////
   //       ,    
   if(m_candles[1].close > m_prev_ma_value && m_candles[0].close < m_ma_value)
      signal = 2;
   //       ,     
   else if (m_candles[1].close < m_prev_ma_value && m_candles[0].close > m_ma_value)
      signal = -2;
      
   ///////////////////////////////////////////////////////////////////
   //                                           //
   ///////////////////////////////////////////////////////////////////
   // ,     
   if (m_candles[0].high - m_candles[0].low >= iVolatilityPercent / 100.0 * m_volatility)
     {
      //    " "
      if (m_candle_pattern < 0 && m_is_highest && m_candles[0].close > m_ma_value)
         signal = -1;
      //    ""
      else if (m_candle_pattern > 0 && m_is_lowest && m_candles[0].close < m_ma_value)
         signal = 1;
     }
     
   return signal;
  }
//+------------------------------------------------------------------+